<?php
if(!defined('DOKU_INC')) die();

if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'action.php');
require_once(DOKU_INC.'inc/JSON.php');

class action_plugin_easydokuwiki extends DokuWiki_Action_Plugin
{

	function getInfo()
	{
		return array(
			'author' => 'neolao',
			'email'  => 'neo@neolao.com',
			'date'   => '2006-10-11',
			'name'   => 'Easy Dokuwiki',
			'desc'   => 'Utils for Easy Dokuwiki Firefox extension',
            'url'    => 'http://resources.neolao.com/xul/easydokuwiki/'
			);
	}

	function register(&$controller)
	{
        $controller->register_hook('ACTION_HEADERS_SEND', 'BEFORE', $this, 'init', 'header');
        $controller->register_hook('ACTION_HEADERS_SEND', 'AFTER', $this, 'init', 'content');
	}

    function init(&$event, $args)
    {
    	// On vérifie qu'on est bien en exécution Easy Dokuwiki
    	if ($_REQUEST['easydokuwiki'] == '1') {
    		// On efface les headers par défaut
    		if ($args == 'header') {
    			$event->data = array();
    		} else {
	    		if (!empty($_REQUEST['u']) && isset($_REQUEST['p'])) {
	    			// Authentification
			    	$this->action_login($event, $args);
	    		} else {
	    			auth_login(null, null);
		    		if ($_REQUEST['ed_action'] == 'logout') {
		    			// Déconnexion
		    			$this->action_logout($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'sitemap') {
		    			// Affichage de la sitemap
		    			$this->action_sitemap($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'pagemap' && !empty($_REQUEST['id'])) {
		    			// Affichage de la pagemap
		    			$this->action_pagemap($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'parse_wiki' && isset($_REQUEST['content'])) {
		    			// Parser du wiki
		    			$this->action_parseWiki($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'save_page' && !empty($_REQUEST['id']) && isset($_REQUEST['content'])) {
		    			// Enregistrer une page
		    			$this->action_savePage($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'delete_page' && !empty($_REQUEST['id'])) {
		    			// Supprimer une page
		    			$this->action_deletePage($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'add_page' && isset($_REQUEST['folder_id']) && !empty($_REQUEST['name'])) {
		    			// Ajouter une page
		    			$this->action_addPage($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'add_folder' && isset($_REQUEST['parent_id']) && !empty($_REQUEST['name'])) {
		    			// Ajouter un dossier
		    			$this->action_addFolder($event, $args);
		    		} elseif ($_REQUEST['ed_action'] == 'quick_acl_check' && !empty($_REQUEST['id'])) {
		    			// Récupérer la permission sur la page
		    			$this->action_quickAclCheck($event, $args);
		    		} else {
		    			// Affichage d'une page
		    			$this->action_page($event, $args);
		    		}
	    		}
    		}
    	}
    }
	
	function action_logout(&$event, $args)
	{
		auth_logoff();
		header('Content-Type: text/plain; charset=UTF-8');
		echo 'logout_ok';
        exit;
	}
	
	function action_login(&$event, $args)
	{
		global $conf;
		
		auth_logoff();
		
		header('Content-Type: text/plain; charset=UTF-8');
		$json = new JSON();
		$infos = array();
		$infos['startPage'] = $conf['start'];
		$infos['userewrite'] = $conf['userewrite'];
		$infos['baseURL'] = DOKU_BASE;
		if (auth_login($_REQUEST['u'], $_REQUEST['p'], true)) {
			$infos['user'] = $_REQUEST['u'];
		}
        echo $json->encode($infos);
        exit;
	}
	
	function action_sitemap(&$event, $args)
	{
		global $conf;
		
		$documents = $this->_sitemap_listFolder(DOKU_INC.'data/pages');
        
        header('Cache-Control: no-cache, must-revalidate');
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
        header('Content-Type: text/xml; charset=UTF-8');
        echo '<?xml version="1.0" encoding="utf-8"?', '>', "\n";
		echo '<RDF:RDF xmlns:RDF="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:WIKI="urn:easydokuwiki:rdf#">';
		
		// La racine du site
		echo '<RDF:Description RDF:about="urn:easydokuwiki:*" WIKI:type="folder" WIKI:acl="', auth_quickaclcheck('*'), '">', "\n";
        echo '<WIKI:title>', $conf['title'], '</WIKI:title>', "\n";
        echo '<WIKI:path>', '', '</WIKI:path>', "\n";
        echo '</RDF:Description>', "\n";
        
		// Le reste du site
		$this->_sitemap_printResourcesDescription($documents);
		
		// Le plan
		echo '<RDF:Seq RDF:about="urn:easydokuwiki:all-documents">';
		
		echo 	'<RDF:li>', "\n";
	    echo 	'<RDF:Seq RDF:about="urn:easydokuwiki:*">', "\n";
		$this->_sitemap_printResourcesMap($documents);
        echo 	'</RDF:Seq>', "\n";
        echo 	'</RDF:li>', "\n";
		
		
		echo '</RDF:Seq>';
		echo '</RDF:RDF>';
		exit;
	}
	
	function action_pagemap(&$event, $args)
	{
		header('Cache-Control: no-cache, must-revalidate');
		header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
		header('Content-Type: text/xml; charset=UTF-8');
        echo $this->_pagemap_get($_REQUEST['id']);
		exit;
	}
	
	function action_page(&$event, $args)
	{
		global $conf;
		
		$id = cleanID($_REQUEST['id']);
        if (empty($id)) {
            $id = $conf['start'];
        }
		
		// Attention, on ne peut pas mettre text/xml parce que le <editor> de XUL ne marche pas avec ce type
        header('Content-Type: text/html; charset=UTF-8');
        //echo '<?xml version="1.0" encoding="utf-8"?', '>', "\n";
        echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">', "\n";
        echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr" lang="fr">';
        echo '<head>';
        echo '<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />';
        echo '<link rel="stylesheet" type="text/css" href="chrome://easydokuwiki/skin/page.css" media="screen"/>';
        echo '<link rel="stylesheet" type="text/css" href="chrome://easydokuwiki/skin/page_print.css" media="print"/>';
        echo '<title>Easy Dokuwiki</title>';
        echo '</head>';
        echo '<body>';
        echo '<!-- ID="', $id, '" -->';
        
        $html = p_wiki_xhtml($id);
        // Suppression du TOC
        $html = preg_replace('@<div class="toc">\n<div class="tocheader toctoggle" id="toc__header">.+?</div>\n</div>\n\n@si', '', $html);
        // Refaire les liens
        $html = preg_replace('@<a href="([^"\?]+)\?([^"]+)" class="wikilink1"@si', '<a href="$1?$2&amp;easydokuwiki=1" class="wikilink1"', $html);
        $html = preg_replace('@<a href="([^"\?]+)\?([^"]+)" class="wikilink2"@si', '<a href="$1?$2&amp;easydokuwiki=1" class="wikilink2"', $html);
        $html = preg_replace('@<a href="([^"\?]+)" class="wikilink1"@si', '<a href="$1&amp;easydokuwiki=1" class="wikilink1"', $html);
        $html = preg_replace('@<a href="([^"\?]+)" class="wikilink2"@si', '<a href="$1&amp;easydokuwiki=1" class="wikilink2"', $html);
        if ($conf['target']['extern'] == '') {
        	$html = preg_replace('@class="urlextern"@si', 'class="urlextern" target="_blank"', $html);
        }
        
        // Si la page n'existe pas, on insère un commentaire
        $file = wikiFN($id);
        if (!@file_exists($file)) {
        	echo '<!-- NEW PAGE -->';
        }
        
        echo $html;
        
        echo '</body>';
        echo '</html>';
        exit;
	}
	
	function action_parseWiki(&$event, $args)
	{
		header('Content-Type: text/plain; charset=UTF-8');
        $html = p_render('xhtml',p_get_instructions($_REQUEST['content']),$info);
        // Suppression du TOC
        $html = preg_replace('@<div class="toc">\n<div class="tocheader toctoggle" id="toc__header">.+?</div>\n</div>\n\n@si', '', $html);
		// Refaire les liens
        
        
        
		echo $html;
		exit;
	}
    
	function action_savePage(&$event, $args)
	{
        global $PRE;
        global $SUF;
        global $SUM;
		header('Content-Type: text/plain; charset=UTF-8');
		
		if (auth_quickaclcheck($_REQUEST['id']) > 1) {
	        saveWikiText($_REQUEST['id'], con($PRE, $_REQUEST['content'], $SUF), $SUM);
	        echo 'save ok';
		} else {
			echo 'no_perm';	
		}
		exit;
	}
	
	function action_deletePage(&$event, $args)
	{
        global $SUM;
		header('Content-Type: text/plain; charset=UTF-8');
		
		if (auth_quickaclcheck($_REQUEST['id']) >= 16) {
        	saveWikiText($_REQUEST['id'], '', $SUM);
	        echo 'delete ok';
		} else {
			echo 'no_perm';
		}
        exit;
	}
	
	function action_addPage(&$event, $args)
	{
		global $SUM;
		
        $folderId = $_REQUEST['folder_id'];
        $pageName = $_REQUEST['name'];
        $pageId = cleanID($pageName);
        $output = array('error'=>'', 'id'=>cleanID($folderId.':'.$pageId));
        
        if (auth_quickaclcheck($output['id']) > 1) {
        	saveWikiText($output['id'], '====== '.$pageName.' ======', $SUM);
        } else {
        	$output['error'] = 'no_perm';
        }
        
        $json = new JSON();
        echo $json->encode($output);
        exit;
	}
	
	function action_quickAclCheck(&$event, $args)
	{
		header('Content-Type: text/plain; charset=UTF-8');
        echo auth_quickaclcheck($_REQUEST['id']);
		exit;
	}
	
	function action_addFolder(&$event, $args)
	{
        $parentId = $_REQUEST['parent_id'];
        $folderName = $_REQUEST['name'];
        $folderId = cleanID($folderName);
        $output = array('error'=>'', 'id'=>cleanID($parentId.':'.$folderId));
        
        if (auth_quickaclcheck($parentId) >= 4) {
        	io_makeFileDir(wikiFN($output['id'].':dummy'));
        } else {
        	$output['error'] = 'no_perm';
        }
        
        $json = new JSON();
        echo $json->encode($output);
        exit;
	}
	
	function _sitemap_listFolder($path, &$docs=array())
	{
	    global $conf;
	    
	    $dir = opendir($path);
	    while ($file = readdir($dir)) {
	      if ($file != "." && $file != "..") {
	        if (is_file($path.'/'.$file)) {
	            // C'est une page
	            if (strpos($file, '.txt.lock') !== false ||
	                strpos($file, '.bak') !== false) {
	                continue;
	            }
	            
	            $pageID = substr($path.'/'.$file, strlen($conf['datadir'])+1);
	            $pageID = str_replace('.txt', '', $pageID); // On retire le .txt
	            $pageID = str_replace('/', ':', $pageID);
	            
	            // Permission
	            $acl = auth_quickaclcheck($pageID);
	            
	            // Vérification des permissions sur le fichier
	            if ($acl > 0) {
	            	// Titre de la page
		            $fileTitle = str_replace('.txt', '', $file);
		            $firstHeading = p_get_first_heading($pageID);
		            if($conf['useheading'] && !empty($firstHeading)){
		                $fileTitle = $firstHeading;
		            }
		            
		            $newfile = array('type'=>'file', 
									 'title'=>$fileTitle, 
									 'path'=>$path.'/'.$file, 
									 'id'=>$pageID, 
									 'acl'=>$acl);
	            
	                $docs[] = $newfile;
	            }
	        } else {
	            // C'est un dossier
	            $folderID = substr($path.'/'.$file, strlen($conf['datadir'])+1);
	            $folderID = str_replace('/', ':', $folderID).':*';
	            
	            // Permission
	            $acl = auth_quickaclcheck($folderID);
	            
	            // Vérification des permissions sur le dossier
	            if ($acl > 0) {
		            $newfolder = array('type'=>'folder', 
									   'title'=>$file, 
									   'list'=>array(), 
									   'path'=>$path.'/'.$file, 
									   'id'=>$folderID, 
									   'acl'=>$acl);
	            
	                $this->_sitemap_listFolder($path.'/'.$file, $newfolder['list']);
	                $docs[] = $newfolder;
	            }
	        }
	      }
	    }
	    closedir($dir);
	    
	    // Tri
	    $type = array();
	    $title = array();
	    foreach ($docs as $key => $row) {
	       $type[$key]  = $row['type'];
	       $title[$key] = $row['title'];
	    }
	    array_multisort($type, SORT_DESC, $title, SORT_ASC, $docs);
	    
	    
	    return $docs;
	}
	function _sitemap_printResourcesDescription($currentFolder, $currentNS='')
	{
	    for($i=0; $i<count($currentFolder); $i++) {
	        if ($currentFolder[$i]['type'] == 'file') { 
	            echo '<RDF:Description RDF:about="urn:easydokuwiki', $currentNS, ':', $currentFolder[$i]['title'], '" WIKI:type="file" WIKI:acl="', $currentFolder[$i]['acl'], '">', "\n";
	            echo '<WIKI:title>', $currentFolder[$i]['title'], '</WIKI:title>', "\n";
	            echo '<WIKI:url>', wl($currentFolder[$i]['id'], '', true), '</WIKI:url>', "\n";
	            echo '<WIKI:path>', $currentFolder[$i]['id'], '</WIKI:path>', "\n";
	            echo '</RDF:Description>', "\n";
	        } else {
	            echo '<RDF:Description RDF:about="urn:easydokuwiki', $currentNS, ':', $currentFolder[$i]['title'], ':*" WIKI:type="folder" WIKI:acl="', $currentFolder[$i]['acl'], '">', "\n";
	            echo '<WIKI:title>', $currentFolder[$i]['title'], '</WIKI:title>', "\n";
	            echo '<WIKI:path>', $currentFolder[$i]['id'], '</WIKI:path>', "\n";
	            echo '</RDF:Description>', "\n";
	            $this->_sitemap_printResourcesDescription($currentFolder[$i]['list'], $currentNS.':'.$currentFolder[$i]['title']);
	        }
	    }
	}
	function _sitemap_printResourcesMap($currentFolder, $currentNS='')
	{
	    
	    for($i=0; $i<count($currentFolder); $i++) {
	        if ($currentFolder[$i]['type'] == 'file') { 
	            echo '<RDF:li RDF:resource="urn:easydokuwiki', $currentNS, ':', $currentFolder[$i]['title'], '"/>', "\n";
	        } else {
	            echo '<RDF:li>', "\n";
	            echo '<RDF:Seq RDF:about="urn:easydokuwiki', $currentNS, ':', $currentFolder[$i]['title'], ':*">', "\n";
	            $this->_sitemap_printResourcesMap($currentFolder[$i]['list'], $currentNS.':'.$currentFolder[$i]['title']);
	            echo '</RDF:Seq>', "\n";
	            echo '</RDF:li>', "\n";
	        }
	    }
	    
	}
	
	function _pagemap_get($id)
	{
	    $page = p_wiki_xhtml($id);
	    $pattern = '@<li class="level(.)"><div class="li"><span class="li"><a href="(.+?)" class="toc">(.+?)</a>@';
	    $matches = null;
	    preg_match_all($pattern, $page, $matches);
	    
	    array_shift($matches);
	    
	    $rdf = '<?xml version="1.0" encoding="utf-8"?'.'>'."\n";
	    $rdf .= '<RDF:RDF xmlns:RDF="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:TOC="urn:easydokuwiki:rdf#">'."\n";
	    
	    for ($i=0; $i<count($matches[0]); $i++) {
	        $rdf .= '<RDF:Description RDF:about="urn:easydokuwiki:'.substr($matches[1][$i], 1).'" TOC:level="'.$matches[0][$i].'">'."\n";
	        $rdf .= '<TOC:title>'.$matches[2][$i].'</TOC:title>'."\n";
	        $rdf .= '<TOC:anchor>'.$matches[1][$i].'</TOC:anchor>'."\n";
	        $rdf .= '</RDF:Description>'."\n";
	    }
	    
	    $rdf .=     '<RDF:Seq RDF:about="urn:easydokuwiki:all-headers">'."\n";
	    
	    $tree = array();
	    $firstLevel = 1;
	    $currentLevel = 0;
	    $nextLevel = 0;
	    for ($i=0; $i<count($matches[0]); $i++) {
	        $currentLevel = (int) $matches[0][$i];
	        if ($i+1 < count($matches[0])) {
	            $nextLevel = (int) $matches[0][$i+1];
	        } else {
	            $nextLevel = $currentLevel;
	        }
	
	        
	        if ($nextLevel <= $currentLevel) {
	            $rdf .= '<RDF:li RDF:resource="urn:easydokuwiki:'.substr($matches[1][$i], 1).'"/>'."\n";
	            $rdf .= str_repeat('</RDF:Seq></RDF:li>'."\n", $currentLevel - $nextLevel);
	        } else {
	            $rdf .= '<RDF:li><RDF:Seq RDF:about="urn:easydokuwiki:'.substr($matches[1][$i], 1).'">'."\n";
	        }
	    }
	    if ($currentLevel > $firstLevel) {
	        $rdf .= str_repeat('</RDF:Seq></RDF:li>'."\n", $currentLevel - $firstLevel);
	    }
	    
	    $rdf .=     '</RDF:Seq>'."\n";
	    $rdf .= '</RDF:RDF>'."\n";
	    
	    return $rdf;
	}
	
}
